#!/usr/bin/env python
# coding: utf-8

# In[31]:


import math

def find_apollonian_circles(given_circles):
    '''Find the circles that touch each of the three input circles.'''
    # Make a list for results.
    solution_circles = []

    # Loop over all of the signs.
    for s0 in (-1, 1):
        for s1 in (-1, 1):
            for s2 in (-1, 1):
                new_circle = find_apollonian_circle(
                    given_circles[0], given_circles[1], given_circles[2],
                    s0, s1, s2)
                if new_circle is not None:
                    solution_circles.append(new_circle)

    # Return the results.
    return solution_circles

# Find a solution to Apollonius' problem.
# For discussion and method, see:
#    http:#mathworld.wolfram.com/ApolloniusProblem.html
#    http:#en.wikipedia.org/wiki/Problem_of_Apollonius#Algebraic_solutions
# For most of a Java code implementation, see:
#    http:#www.diku.dk/hjemmesider/ansatte/rfonseca/implementations/apollonius.html
def find_apollonian_circle(c1, c2, c3, s1, s2, s3):
    # Make sure c2 doesn't have the same X or Y coordinate as the others.
    if math.isclose(c2[0][0], c1[0][0]) or math.isclose(c2[0][1], c1[0][1]):
        c2, c3 = c3, c2
        s2, s3 = s3, s2
    if math.isclose(c2[0][0], c3[0][0]) or math.isclose(c2[0][1], c3[0][1]):
        c2, c1 = c1, c2
        s2, s1 = s1, s2

    assert (
        (not math.isclose(c2[0][0], c1[0][0])) and
        (not math.isclose(c2[0][1], c1[0][1])) and
        (not math.isclose(c2[0][0], c3[0][0])) and
        (not math.isclose(c2[0][1], c3[0][1]))
    ), "Cannot find points without matching coordinates."

    x1 = c1[0][0]
    y1 = c1[0][1]
    r1 = c1[1]
    x2 = c2[0][0]
    y2 = c2[0][1]
    r2 = c2[1]
    x3 = c3[0][0]
    y3 = c3[0][1]
    r3 = c3[1]

    v11 = 2 * x2 - 2 * x1
    v12 = 2 * y2 - 2 * y1
    v13 = x1 * x1 - x2 * x2 + y1 * y1 - y2 * y2 - r1 * r1 + r2 * r2
    v14 = 2 * s2 * r2 - 2 * s1 * r1

    v21 = 2 * x3 - 2 * x2
    v22 = 2 * y3 - 2 * y2
    v23 = x2 * x2 - x3 * x3 + y2 * y2 - y3 * y3 - r2 * r2 + r3 * r3
    v24 = 2 * s3 * r3 - 2 * s2 * r2

    w12 = v12 / v11
    w13 = v13 / v11
    w14 = v14 / v11

    w22 = v22 / v21 - w12
    w23 = v23 / v21 - w13
    w24 = v24 / v21 - w14

    P = -w23 / w22
    Q = w24 / w22
    M = -w12 * P - w13
    N = w14 - w12 * Q

    a = N * N + Q * Q - 1
    b = 2 * M * N - 2 * N * x1 + 2 * P * Q - 2 * Q * y1 + 2 * s1 * r1
    c = x1 * x1 + M * M - 2 * M * x1 + P * P + y1 * y1 - 2 * P * y1 - r1 * r1

    # Find roots of a quadratic equation
    solutions = quadratic_solutions(a, b, c)
    if len(solutions) == 0: return None
    rs = solutions[0]
    xs = M + N * rs
    ys = P + Q * rs

    return ((xs, ys), rs)

def quadratic_solutions(a, b, c):
    '''Return solutions to a quadratic equation.'''
    discriminant = b * b - 4 * a * c

    # See if there are no real solutions.
    if discriminant < 0:
        return ()

    # See if there is one solution.
    if math.isclose(discriminant, 0):
        return (-b / (2 * a),)

    # There are two solutions.
    return (
        (-b + math.sqrt(discriminant)) / (2 * a),
        (-b - math.sqrt(discriminant)) / (2 * a),
    )


# In[32]:


import tkinter as tk
from tkinter import ttk
from PIL import Image, ImageTk
import math

# Layout constants.
MARGIN = 5
WINDOW_WID = 398
WINDOW_HGT = 300

class ApolloniusApp:
    # Create and manage the tkinter interface.
    def __init__(self):
        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('apollonius_problem2')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry(f'{WINDOW_WID}x{WINDOW_HGT}')

        # Build the menus.
        self.build_menus()

        # Build checkboxes.
        chk_frame = tk.Frame(self.window)
        chk_frame.pack(side=tk.LEFT, fill=tk.Y, expand=True)
        self.chk_vars = []
        for i in range(8):
            var = tk.IntVar(self.window, 1)
            chk = tk.Checkbutton(chk_frame, text=f'{i+1}',
                variable=var, command=self.check_changed)
            chk.pack(side=tk.TOP)
            self.chk_vars.append(var)

        btn = tk.Button(chk_frame, text='All', command=self.btn_all_click, width=4)
        btn.pack(side=tk.TOP, padx=5, pady=(10,3))
        btn = tk.Button(chk_frame, text='None', command=self.btn_none_click, width=4)
        btn.pack(side=tk.TOP, padx=5)

        # Build the rest of the UI here...
        self.canvas = tk.Canvas(self.window, bg='white', cursor='cross',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # Create a list to hold three test circles.
        # Each circle looks like ((cx, cy), radius).
        self.circles = []
        self.tangent_circles = []

        # Colors to draw the circles.
        self.fg_colors = [
            'red', 'green', 'blue', 'orange',
            'lime', 'cyan', 'pink', 'purple'
        ]

        # Draw the solution.
        self.draw()

        # Initially we are not drawing a circle.
        self.start_point = None
        self.end_point = None
        self.current_circle = None

        # Make a list to hold two circles.
        # Each circle looks like ((cx, cy), radius).
        self.circles = []

        # Track Button-1 down, Button-1 motion, and Button-1 up.
        self.canvas.bind("<Button-1>", self.mouse_down)
        self.canvas.bind("<B1-Motion>", self.mouse_move)
        self.canvas.bind("<ButtonRelease-1>", self.mouse_up)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)

        self.build_file_menu(menu_bar)

    def build_file_menu(self, menu_bar):
        '''Build the File menu.'''
        file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=file_menu)

        # Exit
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')
        file_menu.add_command(label='Exit', compound='left',
            command=self.mnu_exit, image=self.exit_photo_image)

    def mnu_exit(self):
        self.kill_callback()

    def kill_callback(self):
        self.window.destroy()

    def mnu_clear(self):
        self.canvas.delete(tk.ALL)

    def mouse_down(self, event):
        '''Record the circle's starting point.'''
        self.start_point = (event.x, event.y)

        # If we already have three circles, start over.
        if len(self.circles) == 3:
            self.circles = []
            self.canvas.delete(tk.ALL)

    def mouse_move(self, event):
        # Ignore extraneous events.
        if self.start_point is None: return

        # If there is a current new circle, remove it.
        if self.current_circle is not None:
            self.canvas.delete(self.current_circle)

        # Create the new temporary circle.
        self.end_point = self.get_circle_endpoint(self.start_point, (event.x, event.y))
        self.current_circle = self.canvas.create_oval(
            self.start_point, self.end_point, outline='red')

    def mouse_up(self, event):
        # Ignore extraneous events.
        if self.start_point is None: return

        # If there is a current new circle, remove it.
        if self.current_circle is not None:
            self.canvas.delete(self.current_circle)

        # Create the permanent circle.
        self.end_point = self.get_circle_endpoint(self.start_point, (event.x, event.y))
        self.canvas.create_oval(
            self.start_point, self.end_point, outline='green')

        # Save the new circle's data.
        cx = (self.start_point[0] + self.end_point[0]) / 2
        cy = (self.start_point[1] + self.end_point[1]) / 2
        radius = abs(self.start_point[0] - self.end_point[0]) / 2
        self.circles.append(((cx, cy), radius))

        # If we now have three circles, find their tangent circles.
        if len(self.circles) == 3:
            # Find the tangent circles.
            self.tangent_circles = find_apollonian_circles(self.circles)

            # Draw the new tangent circles.
            self.draw()

        # We are no longer creating a circle.
        self.start_point = None
        self.end_point = None
        self.current_circle = None

    def get_circle_endpoint(self, start_point, end_point):
        '''Return the other corner for the circle.'''
        # Get the width and height.
        dx = end_point[0] - self.start_point[0]
        dy = end_point[1] - self.start_point[1]

        # Use the larger of the width and height.
        if abs(dx) < abs(dy):
            # Make dx have the size of dy but in the direction of dx.
            dx = math.copysign(dy, dx)
        else:
            # Make dy have the size of dx but in the direction of dy.
            dy = math.copysign(dx, dy)
        return (start_point[0] + dx, start_point[1] + dy)

    def check_changed(self):
        self.draw()

    def draw(self):
        self.canvas.delete(tk.ALL)

        # Draw the tangent circles.
        for i in range(len(self.tangent_circles)):
            circle = self.tangent_circles[i]
            if self.chk_vars[i].get():
                fg_color = self.fg_colors[i]
                x = circle[0][0]
                y = circle[0][1]
                radius = circle[1]
                self.canvas.create_oval(
                    x - radius, y - radius,
                    x + radius, y + radius,
                    outline=fg_color
                )

        # Fill the three given circles.
        for circle in self.circles:
            x = circle[0][0]
            y = circle[0][1]
            radius = circle[1]
            self.canvas.create_oval(
                x - radius, y - radius,
                x + radius, y + radius,
                fill='light green'
            )

        # Outline the three given circles.
        for circle in self.circles:
            x = circle[0][0]
            y = circle[0][1]
            radius = circle[1]
            self.canvas.create_oval(
                x - radius, y - radius,
                x + radius, y + radius,
                outline='black'
            )

    def btn_all_click(self):
        '''Check all of the checkboxes.'''
        for var in self.chk_vars:
            var.set(1)
        self.draw()

    def btn_none_click(self):
        '''Uncheck all of the checkboxes.'''
        for var in self.chk_vars:
            var.set(0)
        self.draw()


# In[33]:


ApolloniusApp()
print('Done')


# In[ ]:




